// 9Xtreme
// PPM on PA7
// SPort enable on PB2
// SPort data on PA2

//Taranis JTMS/JTCK usage
//Pads:
//1. Vcc
//2. JTMS - PA13 - use for PDI DATA
//3. JTCK - PA14 - use for PDI CLK
//4. Ground

#include "pdi.h"

#include <string.h>

#define NVM_COMMAND_REG	(XNVM_DATA_BASE+XNVM_CONTROLLER_BASE+XNVM_CONTROLLER_CMD_REG_OFFSET)
#define NVM_CTRLA_REG   (XNVM_DATA_BASE+XNVM_CONTROLLER_BASE+XNVM_CONTROLLER_CTRLA_REG_OFFSET)

uchar PdiSky = 0 ;

const static uint8_t pdi_key[8]={0xFF,0x88,0xD8,0xCD,0x45,0xAB,0x89,0x12};

static void hw_delay( uint16_t time )
{
	delayMicroseconds( time ) ;
}

static void pdiWaitBit()
{
	hw_delay(3) ;	// 3 uS
}

uchar pdiInit()
{
	DDRD |= 0x0C ;
	 
	pdiSetClk1();
	pdiSetData0();
	hw_delay(110) ;	// 110 uS

	pdiSetData1();
//	pdiSetClk1();
	hw_delay(25) ;	// 25uS
	pdiSendIdle();
	pdiSendIdle();

	pdiResetDev(1);

	uchar buf[9];
	buf[0]=XNVM_PDI_KEY_INSTR;
	memcpy(&buf[1],pdi_key,8);
	pdiSendBytes(buf,9);

	pdiSendIdle();

	pdiSendByte(XNVM_PDI_STCS_INSTR | XOCD_CTRL_REGISTER_ADDRESS);
	pdiSendByte(3);

	pdiSendIdle();

	uchar status = pdiWaitNVM() ;
	if (status==PDI_STATUS_OK)
	{
		pdiEnableTimerClock();
	}

	return status;
}

void pdiCleanup()
{
	pdiDisableTimerClock();
	pdiSendIdle();
	pdiResetDev(0);

	DDRD &= ~0x0C ;
	PORTB &= ~0x0C ;
}

void pdiEnableTimerClock()
{
	pdiSetData1();
//	TCCR2=(0<<CS22)|(0<CS21)|(1<<CS20)|(0<<COM21)|(1<<COM20)|(1<<WGM21)|(0<<WGM20);
}

void pdiDisableTimerClock()
{
	pdiSetData1();
//	TCCR2=(0<<CS22)|(0<CS21)|(1<<CS20)|(0<<COM21)|(0<<COM20)|(1<<WGM21)|(0<<WGM20);
}

uchar pdiTimerClockEnabled()
{
//	return (TCCR2 & (1<<COM20));
	return 1 ;
}

void pdiSetClk1()
{
	PORTD |= 0x04 ;
}

void pdiSetClk0()
{
	PORTD &= ~0x04 ;
}

void pdiSetData1()
{
	PORTD |= 0x08 ;
	DDRD |= 0x08 ;
}

void pdiSetData0()
{
	PORTD &= ~0x08 ;
	DDRD |= 0x08 ;
}

void pdiSetDataIn()
{
	DDRD &= ~0x08 ;
	PORTD &= ~0x08 ;
}

uchar pdiGetData()
{
	return PIND & 0x08 ? 1 : 0 ;
}

void pdiSendIdle()
{
	pdiSendByteX(255,255);
}

void pdiSendBreak()
{
	pdiSendByteX(0,0);
}

#define PDI_BIT_START 1
#define PDI_BIT_PARITY 2
#define PDI_BIT_STOP 4

uchar byteParity(uchar b)
{
	b = b ^ (b >> 4) ;
	b = b ^ (b >> 2) ;
	b = b ^ (b >> 1) ;
	return b & 1 ;
}

void pdiSendByte(uchar b)
{
	uchar extra=PDI_BIT_STOP;
	if (byteParity(b)) extra|=PDI_BIT_PARITY;
	pdiSendByteX(b,extra);
}

void pdiSendBytes(uchar* ptr, uint16_t count )
{
	for(;count>0;count--,ptr++)
		pdiSendByte(*ptr);
}

void pdiSendByteX(uchar byte,uchar extra)
{
	pdiSetClk0();
	if (extra&PDI_BIT_START) pdiSetData1(); else pdiSetData0();
	pdiWaitBit();
	pdiSetClk1();
	pdiWaitBit();

	uchar bit;
	for(bit=1;bit;bit<<=1)
	{
		pdiSetClk0();
		if (byte & bit) pdiSetData1(); else pdiSetData0();
		pdiWaitBit();
		pdiSetClk1();
		pdiWaitBit();
	}

	pdiSetClk0();
	if (extra&PDI_BIT_PARITY) pdiSetData1(); else pdiSetData0();
	pdiWaitBit();
	pdiSetClk1();
	pdiWaitBit();

	pdiSetClk0();
	if (extra&PDI_BIT_STOP) pdiSetData1(); else pdiSetData0();
	pdiWaitBit();
	pdiSetClk1();
	pdiWaitBit();

	pdiSetClk0();
	if (extra&PDI_BIT_STOP) pdiSetData1(); else pdiSetData0();
	pdiWaitBit();
	pdiSetClk1();
	pdiWaitBit();
}

uchar pdiReadByte(uchar timeout,uchar *result)
{
	uchar in=PDI_STATUS_TIMEOUT,ret=PDI_STATUS_OK;
	pdiSetDataIn();
	for(;timeout>0;timeout--)
	{
		pdiSetClk0();
//		pdiSetDataIn();
		pdiWaitBit();
		pdiSetClk1();
		in=pdiGetData();
		pdiWaitBit();
		if (in==0) break;
	}
	
	if (in) return PDI_STATUS_TIMEOUT;

	uchar bit,byte=0,parity;
	for(bit=1;bit;bit<<=1)
	{
		pdiSetClk0();
		pdiWaitBit();
		pdiSetClk1();
		if (pdiGetData()) byte|=bit;
		pdiWaitBit();
	}
	*result=byte;

	pdiSetClk0();
	pdiWaitBit();
	pdiSetClk1();
	parity=pdiGetData();
	pdiWaitBit();

	if (parity!=byteParity(byte)) ret=PDI_STATUS_PARITY;

	pdiSetClk0();
	pdiWaitBit();
	pdiSetClk1();
	if ((ret==PDI_STATUS_OK)&&(!pdiGetData())) ret=PDI_STATUS_BADSTOP;
	pdiWaitBit();

	pdiSetClk0();
	pdiWaitBit();
	pdiSetClk1();
	if ((ret==PDI_STATUS_OK)&&(!pdiGetData())) ret=PDI_STATUS_BADSTOP;
	pdiWaitBit();

	return ret;
}

uchar pdiReadCtrl(uint32_t addr, uchar *value)
{
	uchar ret;
	uchar buf[5];

	buf[0]=XNVM_PDI_LDS_INSTR | XNVM_PDI_LONG_ADDRESS_MASK | XNVM_PDI_BYTE_DATA_MASK;
	memmove(buf+1,&addr,4);
	pdiSendBytes(buf,5);
	ret = pdiReadByte(250,value);
	pdiSendBreak();
	pdiSendIdle();
	pdiSendBreak();
	pdiSendIdle();
	return ret;
}

uchar pdiWaitNVM()
{
	uchar retry=100;
	for(;retry>0;retry--)
	{
//	 	wdt_reset() ;
		uchar status;
		if (pdiReadCtrl(XNVM_CONTROLLER_BASE+XNVM_DATA_BASE+XNVM_CONTROLLER_STATUS_REG_OFFSET, &status)==PDI_STATUS_OK)
		{
			if ((status & XNVM_NVM_BUSY)==0)
			{
				return PDI_STATUS_OK;
			}
		}
	}
	return PDI_STATUS_NVM_TIMEOUT;
}

uchar pdiWriteCtrl(uint32_t addr,uint8_t value)
{
	uchar cmd[6];
	cmd[0]= XNVM_PDI_STS_INSTR | XNVM_PDI_LONG_ADDRESS_MASK | XNVM_PDI_BYTE_DATA_MASK;
	memmove(cmd+1,&addr,4);
	cmd[5]=value;
	pdiSendBytes(cmd,6);
	return PDI_STATUS_OK;
}

uchar pdiResetDev(uchar reset)
{
	uchar buf[2];
	buf[0]=XNVM_PDI_STCS_INSTR | XOCD_RESET_REGISTER_ADDRESS;
	buf[1]=reset?XOCD_RESET_SIGNATURE:0;
	pdiSendBytes(buf,2);
	return PDI_STATUS_OK;
}

uchar pdiChipErase()
{
	uint8_t status ;
	uint32_t retryCount ;

	pdiWriteCtrl( NVM_COMMAND_REG, XNVM_CMD_CHIP_ERASE ) ;
	/* Write the CMDEX to execute command */
	pdiWriteCtrl( NVM_CTRLA_REG, XNVM_CTRLA_CMDEX ) ;
	
	/* Wait until the NVM controller is no longer busy */
	retryCount = 10 ;
	while ( retryCount )
	{
		status = pdiWaitNVM() ;
		if (status == PDI_STATUS_OK )
		{
			break ;
		}
		retryCount -= 1 ;
	}
	if (status != PDI_STATUS_OK )
	{
		pdiEnableTimerClock() ;
	}	
	return status ;
}


uchar pdiWriteFuse( uint32_t address, uint8_t value )
{
	uchar cmd[6] ;
	uint32_t register_address;

	pdiWriteCtrl( NVM_COMMAND_REG, XNVM_CMD_WRITE_FUSE ) ;

	cmd[0] = XNVM_PDI_STS_INSTR | XNVM_PDI_LONG_ADDRESS_MASK | XNVM_PDI_BYTE_DATA_MASK ;

	register_address = XNVM_FUSE_BASE + address ;

	memmove((cmd + 1), (uint8_t*)&register_address, 4);
	cmd[5] = value;

	pdiSendBytes(cmd,6);

	uchar status = pdiWaitNVM() ;
	if (status==PDI_STATUS_OK)
	{
		pdiEnableTimerClock();
	}
	return status;
}


uchar pdiSetPointer(uint32_t addr)
{
	uchar cmd[5];
	cmd[0]=XNVM_PDI_ST_INSTR | XNVM_PDI_LD_PTR_ADDRESS_MASK | XNVM_PDI_LONG_DATA_MASK;
	memmove(cmd+1,&addr,4);
	pdiSendBytes(cmd,5);
	return 0;
}

uchar pdiReadBlock(uint32_t addr,uchar* data,uchar len)
{
	uchar ret=PDI_STATUS_OK;

	uchar retry=20;
	for(;retry>0;retry--)
	{
//	 	wdt_reset() ;
		pdiWriteCtrl(XNVM_DATA_BASE+XNVM_CONTROLLER_BASE
		     +XNVM_CONTROLLER_CMD_REG_OFFSET,XNVM_CMD_READ_NVM_PDI);
		pdiSetPointer(addr);

		if (len>1)
		{
			pdiSendByte(XNVM_PDI_REPEAT_INSTR | XNVM_PDI_BYTE_DATA_MASK);
			pdiSendByte(len-1);
		}

		pdiSendByte(XNVM_PDI_LD_INSTR | XNVM_PDI_LD_PTR_STAR_INC_MASK | XNVM_PDI_BYTE_DATA_MASK);

		uchar *dst=data;
		uchar i;
		for(i=0;i<len;i++,dst++)
		{
			ret=pdiReadByte(250,dst);
			if (ret!=PDI_STATUS_OK) break;
		}
		pdiSendBreak();
		pdiSendIdle();
		pdiSendBreak();
		pdiSendIdle();
			if (ret==PDI_STATUS_OK) break;
	}

	return ret;
}


uint8_t pdiWritePageMemory( uint32_t writeAddress, uint8_t* writeBuffer, uint16_t writeSize )
{
	// Wait until the NVM controller is no longer busy
	uint8_t status = pdiWaitNVM() ;
	uint32_t boot = writeAddress >= 0x8000 ? 1 : 0 ;
	if (status != PDI_STATUS_OK )
	{
		pdiEnableTimerClock();
		return status;
	}	

	writeAddress += XNVM_FLASH_BASE ;
	// Send the memory buffer erase command to the target
	pdiSetPointer( 0 ) ;
	pdiWriteCtrl( NVM_COMMAND_REG, XNVM_CMD_ERASE_FLASH_PAGE_BUFFER ) ;

	// Set CMDEX bit in NVM CTRLA register to start the buffer erase
	pdiWriteCtrl( NVM_CTRLA_REG, XNVM_CTRLA_CMDEX ) ;

	if (writeSize)
	{
		/* Wait until the NVM controller is no longer busy */
		status = pdiWaitNVM() ;
		if (status != PDI_STATUS_OK )
		{
			pdiEnableTimerClock() ;
			return status ;
		}	

		// Send the memory buffer write command to the target
		pdiWriteCtrl( NVM_COMMAND_REG, XNVM_CMD_LOAD_FLASH_PAGE_BUFFER ) ;

		// Load the PDI pointer register with the start address we want to write to
		pdiSetPointer( writeAddress ) ;

		// Send the REPEAT command with the specified number of bytes to write
		pdiSendByte(XNVM_PDI_REPEAT_INSTR | XNVM_PDI_BYTE_DATA_MASK);
		pdiSendByte(writeSize-1);

		// Send a ST command with indirect access and post-increment to write the bytes */
		pdiSendByte(XNVM_PDI_ST_INSTR | XNVM_PDI_LD_PTR_STAR_INC_MASK | XNVM_PDI_BYTE_DATA_MASK) ;
		pdiSendBytes( writeBuffer, writeSize ) ;
	}

	status = pdiWaitNVM() ;
	if (status != PDI_STATUS_OK )
	{
		pdiEnableTimerClock() ;
		return status ;
	}	
	// Send the memory write command to the target */
	if ( boot )
	{
		pdiWriteCtrl( NVM_COMMAND_REG, XNVM_CMD_ERASE_AND_WRITE_BOOT_PAGE) ;
	}
	else
	{
		pdiWriteCtrl( NVM_COMMAND_REG, XNVM_CMD_ERASE_AND_WRITE_APP_SECTION) ;
	}
	// Send the address of the first page location to write the memory page */
	pdiWriteCtrl( writeAddress, 0 ) ;
	
	/* Wait until the NVM controller is no longer busy */
	status = pdiWaitNVM() ;
	if (status != PDI_STATUS_OK )
	{
		pdiEnableTimerClock() ;
	}	
	return status ;
}

//Erase and program the flash memory
//1. Erase the flash page buffer.
//i. Use the ST ptr command to set the address 0x00000000.
//ii. Write Erase Flash Page Buffer (0x26) command to the NVM controller.
//iii. Dummy write the flash page buffer with ST *(ptr++) command.
//iv. Polling the NVM busy bit until it has been cleared.

//2. Load the flash page buffer.
//i. Write the Load Flash Page Buffer (0x23) command to the NVM controller.
//ii. Use the ST ptr command to set the start address.
//iii. Set the data length into the repeat counter with REPEAT command.
//iv. Send the ST *(ptr++) command to the PDI controller.
//v. Send the writing data through the PDIBUS.

//3. Write Erase & Write Application Section Page (0x25) command to the NVM controller.
//4. Use the ST ptr command to set the start address.
//5. Dummy write the flash page with ST *(ptr++) command.
//6. Polling the NVM busy bit until it has been cleared.


